from nuclia.sdk.accounts import NucliaAccounts
from nuclia.tests.fixtures import TESTING_ACCOUNT_SLUG


def test_list_accounts(testing_config):
    accounts = NucliaAccounts()
    all_accounts = accounts.list()
    assert TESTING_ACCOUNT_SLUG in [account.slug for account in all_accounts]



from typing import Optional, Union

from nuclia import get_regional_url
from nuclia.data import get_auth, get_async_auth
from nuclia.decorators import account, accounts, zone
from nuclia.sdk.auth import NucliaAuth, AsyncNucliaAuth
from nuclia_models.accounts.backups import (
    BackupCreate,
    BackupCreateResponse,
    BackupResponse,
    BackupRestore,
)
from pydantic import TypeAdapter, BaseModel
from uuid import UUID

BACKUPS_ENDPOINT = "/api/v1/account/{account_id}/backups"
BACKUP_ENDPOINT = "/api/v1/account/{account_id}/backup/{backup_id}"
RESTORE_ENDPOINT = "/api/v1/account/{account_id}/backup/{backup_id}/restore"


class KnowledgeBoxCreated(BaseModel):
    id: str


class NucliaBackup:
    @property
    def _auth(self) -> NucliaAuth:
        auth = get_auth()
        return auth

    @account
    @zone
    def list(
        self, account_id: Optional[str] = None, zone: Optional[str] = None, **kwargs
    ) -> list[BackupResponse]:
        if not zone:
            raise ValueError("zone is required")
        if not account_id:
            raise ValueError("account_id is required")

        path = get_regional_url(zone, BACKUPS_ENDPOINT.format(account_id=account_id))
        data = self._auth._request("GET", path)

        ta = TypeAdapter(list[BackupResponse])
        return ta.validate_python(data)

    @accounts
    @account
    @zone
    def create(
        self,
        backup: Union[dict, BackupCreate],
        account_id: Optional[str] = None,
        zone: Optional[str] = None,
        **kwargs,
    ) -> BackupCreateResponse:
        if not zone:
            raise ValueError("zone is required")
        if not account_id:
            raise ValueError("account_id is required")

        body: BackupCreate
        if isinstance(backup, dict):
            body = BackupCreate.model_validate(backup)
        else:
            body = backup

        path = get_regional_url(zone, BACKUPS_ENDPOINT.format(account_id=account_id))
        data = self._auth._request(
            "POST", path, body.model_dump(mode="json", exclude_unset=True)
        )
        return BackupCreateResponse.model_validate(data)

    @accounts
    @account
    @zone
    def delete(
        self,
        id: Union[str, UUID],
        account_id: Optional[str] = None,
        zone: Optional[str] = None,
        **kwargs,
    ):
        if not zone:
            raise ValueError("zone is required")
        if not account_id:
            raise ValueError("account_id is required")

        path = get_regional_url(
            zone, BACKUP_ENDPOINT.format(account_id=account_id, backup_id=str(id))
        )
        self._auth._request("DELETE", path)

    @accounts
    @account
    @zone
    def restore(
        self,
        restore: BackupRestore,
        backup_id: Union[str, UUID],
        account_id: Optional[str] = None,
        zone: Optional[str] = None,
        **kwargs,
    ) -> KnowledgeBoxCreated:
        if not zone:
            raise ValueError("zone is required")
        if not account_id:
            raise ValueError("account_id is required")

        body: BackupRestore
        if isinstance(restore, dict):
            body = BackupRestore.model_validate(restore)
        else:
            body = restore

        path = get_regional_url(
            zone,
            RESTORE_ENDPOINT.format(account_id=account_id, backup_id=str(backup_id)),
        )
        data = self._auth._request(
            "POST", path, body.model_dump(mode="json", exclude_unset=True)
        )
        return KnowledgeBoxCreated.model_validate(data)


class AsyncNucliaBackup:
    @property
    def _auth(self) -> AsyncNucliaAuth:
        auth = get_async_auth()
        return auth

    @account
    @zone
    async def list(
        self, account_id: Optional[str] = None, zone: Optional[str] = None, **kwargs
    ) -> list[BackupResponse]:
        if not zone:
            raise ValueError("zone is required")
        if not account_id:
            raise ValueError("account_id is required")

        path = get_regional_url(zone, BACKUPS_ENDPOINT.format(account_id=account_id))
        data = await self._auth._request("GET", path)

        ta = TypeAdapter(list[BackupResponse])
        return ta.validate_python(data)

    @accounts
    @account
    @zone
    async def create(
        self,
        backup: Union[dict, BackupCreate],
        account_id: Optional[str] = None,
        zone: Optional[str] = None,
        **kwargs,
    ) -> BackupCreateResponse:
        if not zone:
            raise ValueError("zone is required")
        if not account_id:
            raise ValueError("account_id is required")

        body: BackupCreate
        if isinstance(backup, dict):
            body = BackupCreate.model_validate(backup)
        else:
            body = backup

        path = get_regional_url(zone, BACKUPS_ENDPOINT.format(account_id=account_id))
        data = await self._auth._request(
            "POST", path, body.model_dump(mode="json", exclude_unset=True)
        )
        return BackupCreateResponse.model_validate(data)

    @accounts
    @account
    @zone
    async def delete(
        self,
        id: Union[str, UUID],
        account_id: Optional[str] = None,
        zone: Optional[str] = None,
        **kwargs,
    ):
        if not zone:
            raise ValueError("zone is required")
        if not account_id:
            raise ValueError("account_id is required")

        path = get_regional_url(
            zone, BACKUP_ENDPOINT.format(account_id=account_id, backup_id=str(id))
        )
        await self._auth._request("DELETE", path)

    @accounts
    @account
    @zone
    async def restore(
        self,
        restore: BackupRestore,
        backup_id: Union[str, UUID],
        account_id: Optional[str] = None,
        zone: Optional[str] = None,
        **kwargs,
    ) -> KnowledgeBoxCreated:
        if not zone:
            raise ValueError("zone is required")
        if not account_id:
            raise ValueError("account_id is required")

        body: BackupRestore
        if isinstance(restore, dict):
            body = BackupRestore.model_validate(restore)
        else:
            body = restore

        path = get_regional_url(
            zone,
            RESTORE_ENDPOINT.format(account_id=account_id, backup_id=str(backup_id)),
        )
        data = await self._auth._request(
            "POST", path, body.model_dump(mode="json", exclude_unset=True)
        )
        return KnowledgeBoxCreated.model_validate(data)



import json
from typing import Union

import httpx
import importlib.metadata
import requests
from tabulate import tabulate
from typing import Optional

from nuclia.exceptions import (
    RateLimitError,
    UserTokenExpired,
    DuplicateError,
    InvalidPayload,
)
from nucliadb_models.resource import ResourceList
from nucliadb_models.search import SyncAskResponse
from nuclia.lib.models import ActivityLogsOutput
from nuclia_models.worker.tasks import TaskDefinition, TaskList
from nucliadb_models.resource import KnowledgeBoxList

USER_AGENT = f"nuclia.py/{importlib.metadata.version('nuclia')}"


def handle_http_errors(response: Union[httpx.Response, requests.models.Response]):
    if (
        response.status_code == 403
        and "Hydra token is either unexistent or revoked" in response.text
    ):
        raise UserTokenExpired()
    elif response.status_code == 429:
        raise RateLimitError(f"Rate limited: {response.text}")
    elif response.status_code == 409:
        raise DuplicateError(f"Duplicate resource: {response.text}")
    elif response.status_code == 422:
        raise InvalidPayload(f"Invalid payload: {response.text}")
    elif response.status_code >= 400:
        raise httpx.HTTPError(f"Status code {response.status_code}: {response.text}")


def serialize(obj):
    if isinstance(obj, ResourceList):
        data = []
        for resource in obj.resources:
            status = resource.metadata.status if resource.metadata is not None else ""
            data.append(
                [resource.id, resource.icon, resource.title, status, resource.slug]
            )
        return tabulate(
            data,
            headers=["UUID", "Icon", "Title", "Status", "Slug"],
        )

    if isinstance(obj, KnowledgeBoxList):
        data = []
        for kb in obj.kbs:
            data.append([kb.uuid, kb.slug])
        return tabulate(
            data,
            headers=["UUID", "Slug"],
        )

    if isinstance(obj, TaskDefinition):
        return json.dumps(obj.validation, indent=2)

    if isinstance(obj, TaskList):
        output = ""
        if obj.tasks:
            output += "# Available tasks\n"
            output += "\n"
            data = []
            for task in obj.tasks:
                data.append([task.name, task.description])

            output += tabulate(
                data,
                headers=["Name", "Description"],
            )
            output += "\n\n"

        if obj.running:
            output += "# Running tasks\n"
            output += "\n"
            data = []
            for running_task in obj.running:
                data.append(
                    [
                        running_task.task.name,
                        running_task.id,
                        running_task.scheduled_at,
                    ]
                )

            output += tabulate(
                data,
                headers=["Task", "ID", "Started"],
            )
            output += "\n\n"

        if obj.configs:
            output += "# Configured tasks\n"
            output += "\n"
            data = []
            for running_task in obj.configs:
                data.append(
                    [
                        running_task.task.name,
                        running_task.id,
                    ]
                )

            output += tabulate(
                data,
                headers=["Task", "ID"],
            )
            output += "\n\n"
        return output

    if isinstance(obj, SyncAskResponse):
        if obj.status != "success":
            response = f"ERROR: {obj.error_details}"
        elif obj.answer_json is not None:
            response = f"JSON: {json.dumps(obj.answer_json, indent=2)} \n"
        else:
            response = f"Answer: {obj.answer} \n"

        if obj.metadata:
            response += f"Metadata: {obj.metadata.model_dump_json(indent=2)}\n"

        response += "Resources: \n"
        data = []
        for resource in obj.retrieval_results.resources.values():
            data.append([resource.id, resource.icon, resource.title])
        response += tabulate(
            data,
            headers=["UUID", "Icon", "Title"],
        )
        return response

    if isinstance(obj, ActivityLogsOutput):
        obj = obj.model_dump(exclude_unset=True)

    return obj


def build_httpx_client(
    headers: dict[str, str] = {}, base_url: Optional[str] = None
) -> httpx.Client:
    return httpx.Client(
        headers={"User-Agent": USER_AGENT, **headers}, base_url=(base_url or "")
    )


def build_httpx_async_client(
    headers: dict[str, str] = {}, base_url: Optional[str] = None
) -> httpx.AsyncClient:
    return httpx.AsyncClient(
        headers={"User-Agent": USER_AGENT, **headers}, base_url=(base_url or "")
    )



from nuclia.sdk.kb import NucliaKB, AsyncNucliaKB
from nuclia.tests.fixtures import IS_PROD
from nuclia.lib.kb import LogType
from nuclia_models.events.activity_logs import (
    ActivityLogsQuery,
    Pagination,
    DownloadActivityLogsQuery,
    DownloadFormat,
)
import pytest


def test_logs(testing_config):
    if not IS_PROD:
        assert True
        return
    nkb = NucliaKB()
    logs = nkb.logs.get(type=LogType.NEW, month="2024-06")
    assert len(logs) == 23


def test_activity_logs_query(testing_config):
    if not IS_PROD:
        assert True
        return
    query = ActivityLogsQuery(
        year_month="2024-10",
        show=["id", "date", "client_type", "total_duration"],
        filters={},
        pagination=Pagination(limit=10),
    )
    nkb = NucliaKB()
    output = nkb.logs.query(type=LogType.CHAT, query=query)
    assert len(output.data) == 10
    assert output.has_more


def test_activity_logs_download(testing_config):
    if not IS_PROD:
        assert True
        return
    query = DownloadActivityLogsQuery(
        year_month="2024-10",
        show=["id", "date", "client_type", "total_duration"],
        filters={},
    )
    nkb = NucliaKB()
    output = nkb.logs.download(
        type=LogType.CHAT, query=query, download_format=DownloadFormat.NDJSON
    )
    assert output.request_id
    assert output.download_url is None


@pytest.mark.asyncio
async def test_logs_async(testing_config):
    if not IS_PROD:
        assert True
        return
    nkb = AsyncNucliaKB()
    logs = await nkb.logs.get(type=LogType.NEW, month="2024-06")
    assert len(logs) == 23


@pytest.mark.asyncio
async def test_activity_logs_query_async(testing_config):
    if not IS_PROD:
        assert True
        return
    query = ActivityLogsQuery(
        year_month="2024-10",
        show=["id", "date", "client_type", "total_duration"],
        filters={},
        pagination=Pagination(limit=10),
    )
    nkb = AsyncNucliaKB()
    output = await nkb.logs.query(type=LogType.CHAT, query=query)
    assert len(output.data) == 10
    assert output.has_more


@pytest.mark.asyncio
async def test_activity_logs_download_async(testing_config):
    if not IS_PROD:
        assert True
        return
    query = DownloadActivityLogsQuery(
        year_month="2024-10",
        show=["id", "date", "client_type", "total_duration"],
        filters={},
    )
    nkb = AsyncNucliaKB()
    output = await nkb.logs.download(
        type=LogType.CHAT, query=query, download_format=DownloadFormat.NDJSON
    )
    assert output.request_id
    assert output.download_url is None



from nuclia_models.events.activity_logs import (
    ActivityLogsQueryResponse,
    BaseConfigModel,
    DownloadRequest,
)
from nucliadb_models.metadata import (
    Relation,
    RelationNodeType,
    RelationType,
)
from pydantic import BaseModel
from typing import Optional, Union


class ActivityLogsOutput(BaseConfigModel):
    data: list[ActivityLogsQueryResponse]  # type: ignore
    has_more: bool


DownloadRequestOutput = type(
    "DownloadRequestOutput",
    (BaseModel,),
    {
        "__annotations__": {
            name: field.annotation
            for name, field in DownloadRequest.model_fields.items()
            if name not in {"id", "query", "user_id"}
        }
    },
)


class GraphEntity(BaseModel):
    value: str
    group: Optional[str] = None

    def to_dict(self) -> dict:
        return {
            "value": self.value,
            "group": self.group,
            "type": RelationNodeType.ENTITY,
        }


class GraphRelation(BaseModel):
    label: Optional[str] = None
    source: GraphEntity
    destination: GraphEntity

    def to_relation(self) -> Relation:
        return Relation.parse_obj(
            {
                "from": self.source.to_dict(),
                "to": self.destination.to_dict(),
                "label": self.label,
                "relation": RelationType.ENTITY,
            }
        )


def get_relation(relation: Union[GraphRelation, dict]) -> GraphRelation:
    if isinstance(relation, dict):
        return GraphRelation.parse_obj(relation)
    else:
        return relation



from nuclia.data import get_auth, get_async_auth
from nuclia.decorators import kb
from nuclia.exceptions import InvalidPayload
from nuclia.lib.kb import NucliaDBClient, AsyncNucliaDBClient
from nuclia.sdk.auth import NucliaAuth, AsyncNucliaAuth
from nuclia_models.worker.tasks import (
    ApplyOptions,
    TaskStartKB,
    TaskResponse,
    TaskList,
    TaskName,
    PARAMETERS_TYPING,
    PublicTaskSet,
    TASKS,
)
from typing import Union


class NucliaTask:
    @property
    def _auth(self) -> NucliaAuth:
        auth = get_auth()
        return auth

    @kb
    def list(self, *args, **kwargs) -> TaskList:
        """
        List tasks
        """
        ndb: NucliaDBClient = kwargs["ndb"]
        response = ndb.list_tasks()
        return TaskList.model_validate(response.json())

    @kb
    def start(
        self,
        *args,
        task_name: Union[TaskName, str],
        apply: Union[ApplyOptions, str],
        parameters: Union[PARAMETERS_TYPING, dict],
        **kwargs,
    ) -> TaskResponse:
        """
        Start task

        :param task_name: TaskName enum
        :param apply: EXISTING, NEW, ALL
        :param parameters: Specific parameters depending on the task choosen
        """
        if isinstance(task_name, str):
            task_name = TaskName(task_name.lower())
        if isinstance(apply, str):
            apply = ApplyOptions(apply.upper())
        if isinstance(parameters, dict):
            parameters_model = TASKS[task_name].validation
            if parameters_model is None:
                raise InvalidPayload(f"Invalid parameters for task {task_name.value}")
            parameters = parameters_model.model_validate(parameters)

        ndb: NucliaDBClient = kwargs["ndb"]
        response = ndb.start_task(
            body=TaskStartKB(name=task_name, parameters=parameters, apply=apply)
        )
        return TaskResponse.model_validate(response.json())

    @kb
    def delete(self, *args, task_id: str, **kwargs):
        """
        Delete task

        :param task_id: ID of the task to delete
        """
        ndb: NucliaDBClient = kwargs["ndb"]
        try:
            _ = ndb.delete_task(task_id=task_id)
        except InvalidPayload:
            pass

    @kb
    def stop(self, *args, task_id: str, **kwargs) -> TaskResponse:
        """
        Stop task

        :param task_id: ID of the task to stop
        """
        ndb: NucliaDBClient = kwargs["ndb"]
        response = ndb.stop_task(task_id=task_id)
        return TaskResponse.model_validate(response.json())

    @kb
    def get(self, *args, task_id: str, **kwargs) -> PublicTaskSet:
        """
        Get task

        :param task_id: ID of the task to show
        """
        ndb: NucliaDBClient = kwargs["ndb"]
        response = ndb.get_task(task_id=task_id)
        return PublicTaskSet.model_validate(response.json())

    @kb
    def restart(self, *args, task_id: str, **kwargs) -> TaskResponse:
        """
        Restart task

        :param task_id: ID of the task to restart
        """
        ndb: NucliaDBClient = kwargs["ndb"]
        response = ndb.restart_task(task_id=task_id)
        return TaskResponse.model_validate(response.json())


class AsyncNucliaTask:
    @property
    def _auth(self) -> AsyncNucliaAuth:
        auth = get_async_auth()
        return auth

    @kb
    async def list(self, *args, **kwargs) -> TaskList:
        """
        List tasks
        """
        ndb: AsyncNucliaDBClient = kwargs["ndb"]
        response = await ndb.list_tasks()
        return TaskList.model_validate(response.json())

    @kb
    async def start(
        self,
        *args,
        task_name: Union[TaskName, str],
        apply: Union[ApplyOptions, str],
        parameters: Union[PARAMETERS_TYPING, dict],
        **kwargs,
    ) -> TaskResponse:
        """
        Start task

        :param task_name: TaskName enum
        :param apply: EXISTING, NEW, ALL
        :param parameters: Specific parameters depending on the task choosen
        """
        if isinstance(task_name, str):
            task_name = TaskName(task_name.lower())
        if isinstance(apply, str):
            apply = ApplyOptions(apply.upper())
        if isinstance(parameters, dict):
            parameters_model = TASKS[task_name].validation
            if parameters_model is None:
                raise InvalidPayload(f"Invalid parameters for task {task_name.value}")
            parameters = parameters_model.model_validate(parameters)

        ndb: AsyncNucliaDBClient = kwargs["ndb"]
        response = await ndb.start_task(
            body=TaskStartKB(name=task_name, parameters=parameters, apply=apply)
        )
        return TaskResponse.model_validate(response.json())

    @kb
    async def delete(self, *args, task_id: str, **kwargs):
        """
        Delete task

        :param task_id: ID of the task to delete
        """
        ndb: AsyncNucliaDBClient = kwargs["ndb"]
        try:
            _ = await ndb.delete_task(task_id=task_id)
        except InvalidPayload:
            pass

    @kb
    async def stop(self, *args, task_id: str, **kwargs) -> TaskResponse:
        """
        Stop task

        :param task_id: ID of the task to stop
        """
        ndb: AsyncNucliaDBClient = kwargs["ndb"]
        response = await ndb.stop_task(task_id=task_id)
        return TaskResponse.model_validate(response.json())

    @kb
    async def get(self, *args, task_id: str, **kwargs) -> PublicTaskSet:
        """
        Get task

        :param task_id: ID of the task to show
        """
        ndb: AsyncNucliaDBClient = kwargs["ndb"]
        response = await ndb.get_task(task_id=task_id)
        return PublicTaskSet.model_validate(response.json())

    @kb
    async def restart(self, *args, task_id: str, **kwargs) -> TaskResponse:
        """
        Restart task

        :param task_id: ID of the task to restart
        """
        ndb: AsyncNucliaDBClient = kwargs["ndb"]
        response = await ndb.restart_task(task_id=task_id)
        return TaskResponse.model_validate(response.json())
